@@ -19,6 +19,7 @@ naiveBayes.default <- function(x, y, laplace = 0, ...) {
     ## create tables
     apriori <- table(y)
     tables <- lapply(x, est)
+    isnumeric <- sapply(x, is.numeric)
 
     ## fix dimname names
     for (i in 1:length(tables))
@@ -27,7 +28,8 @@ naiveBayes.default <- function(x, y, laplace = 0, ...) {
 
     structure(list(apriori = apriori,
                    tables = tables,
-                   levels = levels(y),
+                   levels = if (is.logical(y)) c(FALSE, TRUE) else levels(y),
+                   isnumeric = isnumeric,
                    call   = call
                    ),
 
@@ -46,13 +48,13 @@ naiveBayes.formula <- function(formula, data, laplace = 0, ...,
         m$... <- NULL
         m$laplace = NULL
         m$na.action <- na.action
-        m[[1]] <- as.name("model.frame")
+        m[[1L]] <- quote(stats::model.frame)
         m <- eval(m, parent.frame())
         Terms <- attr(m, "terms")
         if (any(attr(Terms, "order") > 1))
             stop("naiveBayes cannot handle interaction terms")
         Y <- model.extract(m, "response")
-        X <- m[,-attr(Terms, "response"), drop = FALSE]
+        X <- m[,attr(Terms, "term.labels"), drop = FALSE]
 
         return(naiveBayes(X, Y, laplace = laplace, ...))
     } else if (is.array(data)) {
@@ -61,7 +63,8 @@ naiveBayes.formula <- function(formula, data, laplace = 0, ...,
         Yind <- which(nam == Yname)
 
         ## Create Variable index
-        deps <- strsplit(as.character(formula)[3], ".[+].")[[1]]
+#        deps <- strsplit(as.character(formula)[3], ".[+].")[[1]]
+        deps <- labels(terms(formula, data = data))
         if (length(deps) == 1 && deps == ".")
             deps <- nam[-Yind]
         Vind <- which(nam %in% deps)
@@ -73,9 +76,13 @@ naiveBayes.formula <- function(formula, data, laplace = 0, ...,
                          (as.numeric(apriori) + laplace * dim(data)[i]))
         names(tables) <- nam[Vind]
 
+        isnumeric = rep(FALSE, length(Vind))
+        names(isnumeric) <- nam[Vind]
+
         structure(list(apriori = apriori,
                        tables = tables,
                        levels = names(apriori),
+                       isnumeric = isnumeric,
                        call   = call
                        ),
 
@@ -106,8 +113,19 @@ predict.naiveBayes <- function(object,
                                ...) {
     type <- match.arg(type)
     newdata <- as.data.frame(newdata)
+
+    ## fix factor levels to be identical with training data
+    for (i in names(object$tables)) {
+        if (!is.null(newdata[[i]]) && !is.numeric(newdata[[i]]))
+            newdata[[i]] <- factor(newdata[[i]], levels = colnames(object$tables[[i]]))
+        if (object$isnumeric[i] != is.numeric(newdata[[i]]))
+            warning(paste0("Type mismatch between training and new data for variable '", i,
+                           "'. Did you use factors with numeric labels for training, and numeric values for new data?"))
+    }
+
     attribs <- match(names(object$tables), names(newdata))
     isnumeric <- sapply(newdata, is.numeric)
+    islogical <- sapply(newdata, is.logical)
     newdata <- data.matrix(newdata)
     L <- sapply(1:nrow(newdata), function(i) {
         ndata <- newdata[i, ]
@@ -119,7 +137,7 @@ predict.naiveBayes <- function(object,
                     msd <- object$tables[[v]]
                     msd[, 2][msd[, 2] <= eps] <- threshold
                     dnorm(nd, msd[, 1], msd[, 2])
-                  } else object$tables[[v]][, nd]
+                  } else object$tables[[v]][, nd + islogical[attribs[v]]]
                   prob[prob <= eps] <- threshold
                   prob
                 }
@@ -136,7 +154,10 @@ predict.naiveBayes <- function(object,
             })
         }
     })
-    if (type == "class")
-        factor(object$levels[apply(L, 2, which.max)], levels = object$levels)
-    else t(L)
+    if (type == "class") {
+        if (is.logical(object$levels))
+            L[2,] > L[1,]
+        else
+            factor(object$levels[apply(L, 2, which.max)], levels = object$levels)
+    } else t(L)
 }
